mod add_remove;
mod assembly;
mod display;
mod outline;

use nalgebra::DVector;
use std::collections::BTreeSet;
use sycamore::prelude::*;

use add_remove::AddRemove;
use assembly::{Assembly, Constraint, Element};
use display::Display;
use outline::Outline;

#[derive(Clone)]
struct AppState {
    assembly: Assembly,
    selection: Signal<BTreeSet<usize>>
}

impl AppState {
    fn new() -> AppState {
        AppState {
            assembly: Assembly::new(),
            selection: create_signal(BTreeSet::default())
        }
    }
}

fn main() {
    sycamore::render(|| {
        let state = AppState::new();
        let key_a = state.assembly.elements.update(
            |elts| elts.insert(
                Element {
                    id: String::from("wing_a"),
                    label: String::from("Wing A"),
                    color: [1.00_f32, 0.25_f32, 0.00_f32],
                    rep: DVector::<f64>::from_column_slice(&[0.5, 0.5, 0.0, 0.5, -0.25]),
                    constraints: BTreeSet::default()
                }
            )
        );
        let key_b = state.assembly.elements.update(
            |elts| elts.insert(
                Element {
                    id: String::from("wing_b"),
                    label: String::from("Wing B"),
                    color: [0.00_f32, 0.25_f32, 1.00_f32],
                    rep: DVector::<f64>::from_column_slice(&[-0.5, -0.5, 0.0, 0.5, -0.25]),
                    constraints: BTreeSet::default()
                },
            )
        );
        state.assembly.elements.update(
            |elts| elts.insert(
                Element {
                    id: String::from("central"),
                    label: String::from("Central"),
                    color: [0.75_f32, 0.75_f32, 0.75_f32],
                    rep: DVector::<f64>::from_column_slice(&[0.0, 0.0, 0.0, 0.4, -0.625]),
                    constraints: BTreeSet::default()
                }
            )
        );
        state.assembly.insert_constraint(Constraint {
            args: (key_a, key_b),
            rep: 0.5
        });
        provide_context(state);
        
        view! {
            div(id="sidebar") {
                AddRemove {}
                Outline {}
            }
            Display {}
        }
    });
}